import os
import numpy as np
from scipy.ndimage import zoom
from scipy.ndimage import distance_transform_edt as dist_field


def mask_interpolation(mask,factor, **kwargs):

    """ Resizing the mask through interpolation by building the distance field.

        Randomly sampling factor and multi-categories are considered. 

        Implemented by Kuan (Kevin) Zhang, Ph.D., Radiology Informatics Laboratory, Mayo Clinic.


        This implementation follows:
        How to properly interpolate masks for deep learning in medical imaging?
 
        Args:
            mask (np.array): The initial mask matrix in the shape: (slices, x, y)
            factor (tuple): The sampling factor of resizing in the shape: (fx, fy, fz)
      
        Output:
            The interpolated mask matrix to return.

    """

     # Check the number of mask types contained in the input.
#    mask_types = np.unique(mask).size -1
    mask_types = int(mask.max())

    if mask_types <1:
        raise ValueError("Input masks need to include more than one category!")

    if mask_types >1:

        # Creat a list for the multi-category lables.
        mask_mul = []


        # Conver the mask values into binary for each type.
        for i in range(mask_types):
            mask_mul.append(np.where(mask == i+1,1,0))

        mask_dist = mask_mul.copy()

        for i in range(mask_types):
            for z in range(mask.shape[-1]):

                # The inner distance field to the edge: 
                dist_field_inner = dist_field(mask_mul[i][:,:,z])

                # The outer distance field to the edge:
                dist_field_outer = dist_field(np.logical_not(mask_mul[i][:,:,z]))

                mask_dist[i][:,:,z] = dist_field_inner - dist_field_outer
      
            mask_dist[i] = zoom(mask_dist[i],factor,order =1)
        #    mask_dist[i] = zoom(mask_dist[i],factor,**kwargs)


        # Apply the threshold on the interpolated mask values.
        # The threshold is set as 0.0.  
        mask_new = np.zeros(mask_dist[0].shape)
        
        for i in range(mask_types):
            mask_dist[i] = np.where(mask_dist[i]>0.0, 1, 0)

        m_index = np.any(np.array(mask_dist)!=0, axis=0)
        mask_new[m_index] = np.argmax(np.array(mask_dist)[:,m_index],axis=0) + 1 
        #    mask_new = np.where(mask_dist[i] >0.0, i+1, 0)

        return mask_new

    else:

        # For the binary mask type.
        mask_dist = np.zeros(mask.shape)

        for z in range(mask.shape[-1]):

            # The inner distance field to the edge:
            dist_field_inner = dist_field(mask[:,:,z])

            # The outer distance field to the edge:
            dist_field_outer = dist_field(np.logical_not(mask[:,:,z]))

            mask_dist[:,:,z] = dist_field_inner - dist_field_outer

        mask_dist = zoom(mask_dist,factor,order =1)
        #mask_dist = zoom(mask_dist,factor,**kwargs)

        return np.where(mask_dist>0.0, 1,0)
        



import os
import numpy as np
from scipy import ndimage, interpolate
from scipy.ndimage import zoom
#import time
#import random
#import math
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras.layers import Conv3D, MaxPooling3D, Conv3DTranspose, concatenate
import tensorflow.keras.backend as K
import urllib.request
import tarfile
#from processing import *


def SOUP_GAN(thicks_ori, Z_FAC,prep_type=0):

    thicks_ori = rescale_img(thicks_ori, max_val= 10000)

    thins = zoom(thicks_ori, (1,1,Z_FAC))
    
    filedir = os.path.dirname(os.path.abspath('__file__'))    

    isdir = os.path.isdir(filedir + '/Thick-to-thin')
    if isdir == False:
        url = 'https://drive.google.com/uc?export=download&id=1PM9I7ccZP4zTocQqIFN5EEdJKdahvp4j'
        urllib.request.urlretrieve(url,filedir+'/Thick-to-thin.tar.bz2')
        tar = tarfile.open(filedir+'/Thick-to-thin.tar.bz2',"r:bz2")
        tar.extractall()
        tar.close()

    isdir = os.path.isdir(filedir + '/Thin-to-thin')
    if isdir == False:
        url = 'https://drive.google.com/uc?export=download&id=10cJLBy7EMXqXHhVvqCXLc5fKZpooy9w5'
        urllib.request.urlretrieve(url,filedir+'/Thin-to-thin.tar.bz2')
        tar = tarfile.open(filedir+'/Thin-to-thin.tar.bz2',"r:bz2")
        tar.extractall()
        tar.close()



    if prep_type == 0:
        new_model=keras.models.load_model(filedir + '/Thick-to-thin')
    elif prep_type == 1:
        new_model=keras.models.load_model(filedir + '/Thin-to-thin')
    thins_gen = thins.copy()

    target = np.moveaxis(thins,-1,0)
    target = target [...,np.newaxis]
    target = target [np.newaxis,...]

    index  = attention_coeff(target, Z_FAC)

    target = new_model.predict([target,index])
     
    target_small = target[0,...,0]
    thins_gen = np.moveaxis(target_small, 0,-1)

    return thins_gen
     
def attention_coeff(target, Z):
    index  = np.zeros((target.shape[0],5))
    if Z < 2:
        index[:,0] =1
    elif Z < 3:
        t = Z-2
        index[:,0] = 1-t
        index[:,1] = t
    elif Z < 4:
        t = Z-3
        index[:,1] = 1-t
        index[:,2] = t
    elif Z < 5:
        t = Z-4
        index[:,2] = 1-t
        index[:,3] = t
    elif Z < 6:
        t = Z-5
        index[:,3] = 1-t
        index[:,4] = t
    else:
        index[:,4] = 1
    return index

MAX_VAL = 10000

def rescale_img(image, max_val=MAX_VAL):
    image = image - np.min(image)
    image = (np.maximum(image, 0) / image.max()) * max_val
    return (image)




import setuptools

with open("README.txt", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="KevinSR",
    version="0.1.19",
    author="Kevin Zhang",
    author_email="zhangkuanmayo@gmail.com",
    description="This package includes inference codes supporting Super-resolution image and mask interpolations.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://pypi.org/project/KevinSR",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pydicom>=2.1.2",
        "numpy>=1.18.5",
        "matplotlib>=3.4.2",
        "scipy>=1.4.1",
        "tensorflow-gpu>=2.3.0",
        "pillow>=8.0.0",
    ],
    python_requires='>=3.6',
)
